@@ -485,15 +485,15 @@ if json_ok():
             tj = TestingJson.select().where(TestingJson.data == {'foo': 'bar'})
             sql, params = tj.sql()
             self.assertEqual(sql, (
-                'SELECT t1."id", t1."data" '
-                'FROM "testingjson" AS t1 WHERE (t1."data" = %s)'))
+                'SELECT "t1"."id", "t1"."data" '
+                'FROM "testingjson" AS t1 WHERE ("t1"."data" = %s)'))
             self.assertEqual(params[0].adapted, {'foo': 'bar'})
 
             tj = TestingJson.select().where(TestingJson.data['foo'] == 'bar')
             sql, params = tj.sql()
             self.assertEqual(sql, (
-                'SELECT t1."id", t1."data" '
-                'FROM "testingjson" AS t1 WHERE (t1."data"->>%s = %s)'))
+                'SELECT "t1"."id", "t1"."data" '
+                'FROM "testingjson" AS t1 WHERE ("t1"."data"->>%s = %s)'))
             self.assertEqual(params, ['foo', 'bar'])
 
         def assertItems(self, where, *items):
